/***************************************************************************
 *   Copyright (C) 2005 by Nicolas Brodu                                   *
 *   nicolas.brodu@free.fr                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef QSETTINGS_H
#define QSETTINGS_H

/***************************************************************************
    These settings define the number of Q values, and their relation to the
    array indices.

    Please review these settings with attention.

    You should only need modify this file if you wish to change the Q
    settings. Please also check to ReducedNQ example in that case.
 ***************************************************************************/

// Number of Q values, and array initializer
static const int NUM_Q = 20;
#define EMPTY_Q_INITIALIZER {0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.}

// Correspondance between array index and q value
inline int GET_Q_FOR_INDEX(int i) {
    return (i>=10) ? (9 - i) : (i + 1);
}

// Define this flag if some of the Q values are negative
// See plusExponentiationNeg and minusExponentiationNeg
#define HAS_NEGATIVE_Q 1


// This routine adds up term^q, for q>0 to the power sums
// Note: the reason why positive and negative q are separated is
//       the array indexation. See plusExponentiationNeg
// Perhaps assembly could be used to gain a few % perf
inline void plusExponentiationPos(double term, double* powerSum) {

    const double p1 = term;
    powerSum[0] += term;        // Array index 0 is q=1
    term *= term;
    const double p2 = term;
    powerSum[1] += term;        // ...
    term *= term;
    const double p4 = term;
    powerSum[3] += term;
    powerSum[7] += term * term;

    term = p1 * p2;
    const double p3 = term;
    powerSum[2] += term;
    term *= term;
    powerSum[5] += term;
    powerSum[8] += term * p3;

    term = p3 * p2;
    powerSum[4] += term;
    powerSum[9] += term * term;

    powerSum[6] += p3 * p4;
}

// This routine subtracts term^q, for q>0 to the power sums
// Note: the reason why positive and negative q are separated is
//       the array indexation. See minusExponentiationNeg
// Perhaps assembly could be used to gain a few % perf
inline void minusExponentiationPos(double term, double* powerSum) {

    const double p1 = term;
    powerSum[0] -= term;
    term *= term;
    const double p2 = term;
    powerSum[1] -= term;
    term *= term;
    const double p4 = term;
    powerSum[3] -= term;
    powerSum[7] -= term * term;

    term = p1 * p2;
    const double p3 = term;
    powerSum[2] -= term;
    term *= term;
    powerSum[5] -= term;
    powerSum[8] -= term * p3;

    term = p3 * p2;
    powerSum[4] -= term;
    powerSum[9] -= term * term;

    powerSum[6] -= p3 * p4;
}

#ifdef HAS_NEGATIVE_Q

// Does for negative q what plusExponentiationPos does for positive q
// It's up to your program to decide how many array indices are allocated
// to positive and negative q
// In this example, with the same number of allocation, it's easy to
// re-use plusExponentiationPos
inline void plusExponentiationNeg(double term, double* powerSum) {
    plusExponentiationPos(term, powerSum + 10);
}

// See plusExponentiationNeg
inline void minusExponentiationNeg(double term, double* powerSum) {
    minusExponentiationPos(term, powerSum + 10);
}

#endif

/***************************************************************************
    End of the Q dependent settings
 ***************************************************************************/

#endif
