/***************************************************************************
 *   Copyright (C) 2005 by Nicolas Brodu                                   *
 *   nicolas.brodu@free.fr                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
using namespace std;

#include <unistd.h>
#include <math.h>

#include "Analyzer.h"

#ifndef QSETTINGS
#include "QSettings.h"
#else
#include QSETTINGS
#endif

void help(char *argv[], int ret) {
cout <<
"Usage: " << argv[0] << " [options] dataFile" << endl << endl << 

"Apply incremental multifractal analysis on a data series." << endl << endl <<

"Options:" << endl << 
"  -w X       Use Daubechies wavelet with X coefficients. Only 4 & 6 supported in this version. Default is 6." << endl <<
"  -l L       Use L number of levels of decomposition. Default is 7." << endl <<
"  -n N       Use N data at the highest level of decomposition. Default is 5 (see below)." << endl <<
"  -o         Default is to keep old points. Use this to remove them." << endl <<
"  -a         Default is to average results over all dyadic frames. Use this not to average." << endl <<
"  -i         Default is to use the series as is. Use this to integrate the series." << endl <<endl <<

"A dataFile argument is needed. The file format is ascii (not binary), one value per line, without comments at the beginning." << endl <<
"Example:" << endl <<
"  -123.456" << endl <<
"  7.8e9" << endl <<
"... and so on" <<endl << endl <<

"Note that when keeping old values, precision depends only on the number of levels. In that case, there should be at least the wavelet filter size - 1 data at the highest level (max fluctuation filter size). When discarding old values, N should be set to a reasonable value to make statistics on (20, 30,... larger is better)."<< endl;
exit(ret);
}

int main(int argc, char** argv) {

    Wavelet* wavelet = 0;
    int nlevel = 7;
    int nmin = 6;
    bool discardOldPoints = false;
    bool average = true;
    bool integrate = false;
    int option;
    while ((option=getopt(argc, argv, "w:l:n:oai"))!=-1) {
        switch (option) {
            case 'w':
                if (atoi(optarg)==4) wavelet=DW4::getInstance();
                else if (atoi(optarg)==6) wavelet=DW6::getInstance();
                break;
            case 'l':
                nlevel = atoi(optarg);
                break;
            case 'n':
                nmin = atoi(optarg);
                break;
            case 'o':
                discardOldPoints = true;
                break;
            case 'a':
                average = false;
                break;
            case 'i':
                integrate = true;
                break;
            case '?':
                help(argv, 0);
        }
    }
    if (wavelet==0) wavelet=DW6::getInstance();
    if (optind>=argc) help(argv, 0);
    ifstream file;
    file.open(argv[optind]);
    if (!file) {
        cout << "Error: can't read file " << argv[optind] << endl;
        help(argv, 1);
    }
    
    cout << "Processing data with:" << endl;
    cout << ". Daubechies wavelet with " << wavelet->support << " coefficients." << endl;
    cout << ". " << nlevel << " number of levels of decomposition." << endl;
    cout << ". " << nmin << " data at the highest level." << endl;
    cout << ". Old points are " << (discardOldPoints?"discarded":"kept") << "." <<endl;
    cout << ". Averaging is " << (average?"done":"not done") << " over dyadic frames." <<endl;
    cout << ". The data series is " << (integrate?"integrated":"used directly") << "." <<endl;

// Uncomment this to set different weights for the weighted least square fitting (exponential data fit).
// See the article: wrong weights (or no weight at all) just give less precise results, not completely wrong ones.
// Default weights are suitable for monofractal H=0.5 (brownian motion).
/*
    // Array of size the number of levels
    double weights[7];
    // loop on all scales
    for (int level=1; level<=7; ++level) {
        // scale s=2^level. weight = scale^H.
        // These weights are adapted to H=1.0 (example: integrated pink noise).
        weights[level-1] = (double)(1 << level);
    }
*/

    Analyzer analyzer(nlevel, nmin, wavelet /*, weights*/, discardOldPoints, average);

    cout << endl<< "With these settings, at least " << analyzer.data.capacity() << " data are needed in the file. This number depends on the wavelet filter size (-w), the number of levels (-l), and the higest level size (-n)." << endl << endl;

    DecompositionLevel* level;

    // may not be used
    double integ = 0.0;

    // loop over data in the file
    while (file.good()) {
        double value;
        file >> value;
        // integrate if requested
        if (integrate) {
            integ += value;
            value = integ;
        }
        analyzer.newDataMFA(value);
    }
    file.close();

    if (!analyzer.data.isFull()) {
        cout << "Not enough data." << endl;
        return 2;
    }

    cout << "Results:" << endl;
    cout.setf(ios_base::left);
    cout << setw(6) << "q" << setw(12) << "h(q)" << setw(12) <<"correlation coef."<<endl;
    for (int i=0; i<NUM_Q; ++i) {
        cout << setw(6);
        cout << GET_Q_FOR_INDEX(i);
        cout << setw(12) << analyzer.hq[i];
        cout << setw(12) << sqrt(analyzer.squaredCorrelation[i]) << endl;
    }

    return 0;
}
