/***************************************************************************
 *   Copyright (C) 2005 by Nicolas Brodu                                   *
 *   nicolas.brodu@free.fr                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <iostream>
#include <fstream>
#include <sstream>
using namespace std;

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "Analyzer.h"
#include "Utility.h"

int main(int argc, const char** argv) {

    // Process experiment with different random seeds
    // for enough independent random runs so we can compute
    // mean, variation, and derive error bars if necessary.
    // Set the seeds explicitly so the experiment is reproducible.

    // Arrays to store statistics
    double averageStepsToDiff[20];
    double stddevStepsToDiff[20];

    // double 0.0 is 0-bitfield in IEEE standard
    memset(averageStepsToDiff, 0, 20*sizeof(double));
    memset(stddevStepsToDiff, 0, 20*sizeof(double));

    const int Nruns = 30;
    Wavelet* wavelet = DW6::getInstance();
    // pre-allocate filters for all loops
    ScaleDeltaFilterArray::getInstance(wavelet, 5);
    
    // Use low and high bits for the seed
    for (int run =0; run < Nruns; ++run) {
        Utility::RandomInit(1L<<run);

        // Default analyzer has the correct H=0.5 weights already
        // so as to avoid any scale bias for Brownian motion
        // Remove old points, but do not average over 2^L frames
        Analyzer analyzer(5, 10, wavelet, true, false);
        Analyzer analyzerFull(5, 10, wavelet, true, false);

        long stepsToDiff[20];
        memset(stepsToDiff, 0, 20*sizeof(int));

        // trapezoidal integration (minus first and last terms)
        // is just adding terms
        double fbm = 0.0;

        cout << "Running MFA analysis on brownian motion..." << endl;
        for (long i=0; i<10000000; ++i) {
            fbm += Utility::NRandom(); // gaussian white noise
            // Run one analyzer in incremental mode
            analyzer.newDataMFA(fbm);
            // Apply non-incremental MFA to the other each update (after init phase)
            if (analyzerFull.newDataMFA(fbm)==0) analyzerFull.nonIncrementalMFA();
            // Monitor time to see difference
            int ndiff=0;
            for (int j=0; j<20; ++j) {
                if (stepsToDiff[j]!=0) {++ndiff; continue;}
                if (fabs(analyzer.hq[j] - analyzerFull.hq[j])>1e-6) stepsToDiff[j] = i;
            }
            if (ndiff==20) break;
            if ((i&0xFFF)==0) {
                cout << "partial results for i=" << i << endl;
                for (int j=0; j<10; ++j) cout << "n[" << (j+1) << "]=" << stepsToDiff[j] << " ";
                for (int j=10; j<20; ++j) cout << "n[" << (9-j) << "]=" << stepsToDiff[j] << " ";
                cout << endl;
            }
        }
    
        cout << "Incremental version: " << endl;
        for (int i=0; i<10; ++i) cout << "h(" << (i+1) << ")=" << analyzer.hq[i] << " ";
        for (int i=10; i<20; ++i) cout << "h(" << (9-i) << ")=" << analyzer.hq[i] << " ";
        cout << endl;
        cout << "Full version: " << endl;
        for (int i=0; i<10; ++i) cout << "h(" << (i+1) << ")=" << analyzerFull.hq[i] << " ";
        for (int i=10; i<20; ++i) cout << "h(" << (9-i) << ")=" << analyzerFull.hq[i] << " ";
        cout << endl;
        cout << "Number of steps to see 1e-6 difference: " << endl;
        for (int i=0; i<10; ++i) cout << "n[" << (i+1) << "]=" << stepsToDiff[i] << " ";
        for (int i=10; i<20; ++i) cout << "n[" << (9-i) << "]=" << stepsToDiff[i] << " ";
        cout << endl;
        
        for (int i=0; i<20; ++i) {
            averageStepsToDiff[i] += stepsToDiff[i];
            stddevStepsToDiff[i] += (double)stepsToDiff[i] * (double)stepsToDiff[i];
        }
    }

    // complete statistics
    for (int i=0; i<20; ++i) {
        averageStepsToDiff[i] /= Nruns;
        stddevStepsToDiff[i] = sqrt((stddevStepsToDiff[i] / Nruns) - (averageStepsToDiff[i] * averageStepsToDiff[i]));
    }

    cout << "Average number of steps to see 1e-6 difference: " << endl;
    for (int i=0; i<10; ++i) cout << "n[" << (i+1) << "]=" << averageStepsToDiff[i] << " ";
    for (int i=10; i<20; ++i) cout << "n[" << (9-i) << "]=" << averageStepsToDiff[i] << " ";
    cout << endl;

    cout << "Standard deviation in number of steps to see 1e-6 difference: " << endl;
    for (int i=0; i<10; ++i) cout << "d[" << (i+1) << "]=" << stddevStepsToDiff[i] << " ";
    for (int i=10; i<20; ++i) cout << "d[" << (9-i) << "]=" << stddevStepsToDiff[i] << " ";
    cout << endl;
}
